@@ -11,8 +11,7 @@
 
 import click
 
-from aiida.cmdline.commands.cmd_data import verdi_data
-from aiida.cmdline.commands.cmd_data import cmd_show
+from aiida.cmdline.commands.cmd_data import cmd_show, verdi_data
 from aiida.cmdline.commands.cmd_data.cmd_export import data_export, export_options
 from aiida.cmdline.commands.cmd_data.cmd_list import data_list, list_options
 from aiida.cmdline.params import arguments, options, types
@@ -34,9 +33,10 @@ def cif():
 @decorators.with_dbenv()
 def cif_list(raw, formula_mode, past_days, groups, all_users):
     """List store CifData objects."""
-    from aiida.orm import CifData
     from tabulate import tabulate
 
+    from aiida.orm import CifData
+
     elements = None
     elements_only = False
 
@@ -45,14 +45,14 @@ def cif_list(raw, formula_mode, past_days, groups, all_users):
     )
 
     counter = 0
-    cif_list_data = list()
+    cif_list_data = []
 
     if not raw:
         cif_list_data.append(LIST_PROJECT_HEADERS)
     for entry in entry_list:
         for i, value in enumerate(entry):
             if isinstance(value, list):
-                new_entry = list()
+                new_entry = []
                 for elm in value:
                     if elm is None:
                         new_entry.append('')
@@ -71,7 +71,7 @@ def cif_list(raw, formula_mode, past_days, groups, all_users):
 
 
 @cif.command('show')
-@arguments.DATA(type=types.DataParamType(sub_classes=('aiida.data:cif',)))
+@arguments.DATA(type=types.DataParamType(sub_classes=('aiida.data:core.cif',)))
 @options.VISUALIZATION_FORMAT(type=click.Choice(VISUALIZATION_FORMATS), default='jmol')
 @decorators.with_dbenv()
 def cif_show(data, fmt):
@@ -85,7 +85,7 @@ def cif_show(data, fmt):
 
 
 @cif.command('content')
-@arguments.DATA(type=types.DataParamType(sub_classes=('aiida.data:cif',)))
+@arguments.DATA(type=types.DataParamType(sub_classes=('aiida.data:core.cif',)))
 @decorators.with_dbenv()
 def cif_content(data):
     """Show the content of the CIF file."""
@@ -97,7 +97,7 @@ def cif_content(data):
 
 
 @cif.command('export')
-@arguments.DATUM(type=types.DataParamType(sub_classes=('aiida.data:cif',)))
+@arguments.DATUM(type=types.DataParamType(sub_classes=('aiida.data:core.cif',)))
 @options.EXPORT_FORMAT(type=click.Choice(EXPORT_FORMATS), default='cif')
 @export_options
 @decorators.with_dbenv()
@@ -124,4 +124,4 @@ def cif_import(filename):
         node, _ = CifData.get_or_create(filename)
         echo.echo_success(f'imported {str(node)}')
     except ValueError as err:
-        echo.echo_critical(err)
+        echo.echo_critical(str(err))
