@@ -8,15 +8,27 @@
 # For further information please visit http://www.aiida.net               #
 ###########################################################################
 """Click parameter type for AiiDA Plugins."""
+import functools
 
 import click
+from importlib_metadata import EntryPoint
 
-from aiida.cmdline.utils import decorators
 from aiida.common import exceptions
-from aiida.plugins.entry_point import ENTRY_POINT_STRING_SEPARATOR, ENTRY_POINT_GROUP_PREFIX, EntryPointFormat
-from aiida.plugins.entry_point import format_entry_point_string, get_entry_point_string_format
-from aiida.plugins.entry_point import get_entry_point, get_entry_points, get_entry_point_groups
-from ..types import EntryPointType
+from aiida.plugins import factories
+from aiida.plugins.entry_point import (
+    ENTRY_POINT_GROUP_PREFIX,
+    ENTRY_POINT_STRING_SEPARATOR,
+    EntryPointFormat,
+    format_entry_point_string,
+    get_entry_point,
+    get_entry_point_groups,
+    get_entry_point_string_format,
+    get_entry_points,
+)
+
+from .strings import EntryPointType
+
+__all__ = ('PluginParamType',)
 
 
 class PluginParamType(EntryPointType):
@@ -38,6 +50,18 @@ class PluginParamType(EntryPointType):
     """
     name = 'plugin'
 
+    _factory_mapping = {
+        'aiida.calculations': factories.CalculationFactory,
+        'aiida.data': factories.DataFactory,
+        'aiida.groups': factories.GroupFactory,
+        'aiida.parsers': factories.ParserFactory,
+        'aiida.schedulers': factories.SchedulerFactory,
+        'aiida.transports': factories.TransportFactory,
+        'aiida.tools.dbimporters': factories.DbImporterFactory,
+        'aiida.tools.data.orbitals': factories.OrbitalFactory,
+        'aiida.workflows': factories.WorkflowFactory,
+    }
+
     def __init__(self, group=None, load=False, *args, **kwargs):
         """
         Validate that group is either a string or a tuple of valid entry point groups, or if it
@@ -135,15 +159,15 @@ class PluginParamType(EntryPointType):
 
         return possibilites
 
-    def complete(self, ctx, incomplete):  # pylint: disable=unused-argument
+    def shell_complete(self, ctx, param, incomplete):  # pylint: disable=unused-argument
         """
         Return possible completions based on an incomplete value
 
         :returns: list of tuples of valid entry points (matching incomplete) and a description
         """
-        return [(p, '') for p in self.get_possibilities(incomplete=incomplete)]
+        return [click.shell_completion.CompletionItem(p) for p in self.get_possibilities(incomplete=incomplete)]
 
-    def get_missing_message(self, param):
+    def get_missing_message(self, param):  # pylint: disable=unused-argument
         return 'Possible arguments are:\n\n' + '\n'.join(self.get_valid_arguments())
 
     def get_entry_point_from_string(self, entry_point_string):
@@ -167,50 +191,65 @@ class PluginParamType(EntryPointType):
             if entry_point_format == EntryPointFormat.PARTIAL:
                 group = ENTRY_POINT_GROUP_PREFIX + group
 
-            if group not in self.groups:
-                raise ValueError('entry point group {} is not supported by this parameter')
+            self.validate_entry_point_group(group)
 
         elif entry_point_format == EntryPointFormat.MINIMAL:
 
             name = entry_point_string
-            matching_groups = [group for group, entry_point in self._entry_points if entry_point.name == name]
+            matching_groups = {group for group, entry_point in self._entry_points if entry_point.name == name}
 
             if len(matching_groups) > 1:
                 raise ValueError(
                     "entry point '{}' matches more than one valid entry point group [{}], "
-                    'please specify an explicit group prefix'.format(name, ' '.join(matching_groups))
+                    'please specify an explicit group prefix: {}'.format(
+                        name, ' '.join(matching_groups), self._entry_points
+                    )
                 )
             elif not matching_groups:
                 raise ValueError(
                     "entry point '{}' is not valid for any of the allowed "
                     'entry point groups: {}'.format(name, ' '.join(self.groups))
                 )
-            else:
-                group = matching_groups[0]
+
+            group = matching_groups.pop()
 
         else:
             ValueError(f'invalid entry point string format: {entry_point_string}')
 
+        # If there is a factory for the entry point group, use that, otherwise use ``get_entry_point``
         try:
-            entry_point = get_entry_point(group, name)
+            get_entry_point_partial = functools.partial(self._factory_mapping[group], load=False)
+        except KeyError:
+            get_entry_point_partial = functools.partial(get_entry_point, group)
+
+        try:
+            return get_entry_point_partial(name)
         except exceptions.EntryPointError as exception:
             raise ValueError(exception)
 
-        return entry_point
+    def validate_entry_point_group(self, group):
+        if group not in self.groups:
+            raise ValueError(f'entry point group `{group}` is not supported by this parameter.')
 
-    @decorators.with_dbenv()
     def convert(self, value, param, ctx):
         """
         Convert the string value to an entry point instance, if the value can be successfully parsed
         into an actual entry point. Will raise click.BadParameter if validation fails.
         """
-        value = super().convert(value, param, ctx)
+        # If the value is already of the expected return type, simply return it. This behavior is new in `click==8.0`:
+        # https://click.palletsprojects.com/en/8.0.x/parameters/#implementing-custom-types
+        if isinstance(value, EntryPoint):
+            try:
+                self.validate_entry_point_group(value.group)
+            except ValueError as exception:
+                raise click.BadParameter(str(exception))
+            return value
 
-        if not value:
-            raise click.BadParameter('plugin name cannot be empty')
+        value = super().convert(value, param, ctx)
 
         try:
             entry_point = self.get_entry_point_from_string(value)
+            self.validate_entry_point_group(entry_point.group)
         except ValueError as exception:
             raise click.BadParameter(str(exception))
 
