@@ -88,6 +88,11 @@ class JobResource(DefaultFieldsAttributeDict, metaclass=abc.ABCMeta):
     def accepts_default_mpiprocs_per_machine(cls):
         """Return True if this subclass accepts a `default_mpiprocs_per_machine` key, False otherwise."""
 
+    @classmethod
+    def accepts_default_memory_per_machine(cls):
+        """Return True if this subclass accepts a `default_memory_per_machine` key, False otherwise."""
+        return True
+
     @abc.abstractmethod
     def get_tot_num_mpiprocs(self):
         """Return the total number of cpus of this job resource."""
