@@ -16,7 +16,7 @@ from sqlalchemy.util.compat import nullcontext
 from aiida.cmdline import is_verbose
 from aiida.cmdline.groups.verdi import VerdiCommandGroup
 from aiida.cmdline.params import options
-from aiida.storage.psql_dos.migrator import PsqlDostoreMigrator
+from aiida.storage.psql_dos.migrator import PsqlDosMigrator
 
 
 class AlembicRunner:
@@ -33,7 +33,7 @@ class AlembicRunner:
         """
         if self.profile is None:
             raise click.ClickException('No profile specified')
-        migrator = PsqlDostoreMigrator(self.profile)
+        migrator = PsqlDosMigrator(self.profile)
 
         context = migrator._alembic_connect() if connect else nullcontext(migrator._alembic_config())  # pylint: disable=protected-access
         with context as config:
